#!/usr/bin/env python
# -*- coding: ascii -*-
from __future__ import absolute_import
from __future__ import print_function

r"""
A tkinter code that helps users digitize curves from plot images.

If a 2D plot is available only on paper or an image file, but the user needs the x,y data pairs in digital form, DigiPlot will help to create those x,y data pairs.


DigiPlot
Copyright (C) 2017  Charlie Taylor

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

-----------------------

"""
import os
here = os.path.abspath(os.path.dirname(__file__))
import sys
if sys.version_info < (3,):
    from future import standard_library
    standard_library.install_aliases()
    import tkFileDialog
    import tkMessageBox
else:
    import tkinter.filedialog as tkFileDialog
    import tkinter.messagebox as tkMessageBox

#  Need to redefine object here
# pylint: disable=W0622, W0703, W0612, W0403, C0330
from builtins import object

import tkinter.font
#from tkinter import *
from tkinter import Menu, StringVar, Label, SUNKEN, SW, X, BOTTOM, Frame, NE, NW,\
    BOTH, TOP, Button, LEFT, SE, Scrollbar, VERTICAL, Text, RIGHT, Y, END, Tk,\
    Canvas, Listbox, Entry, N, S, E, W, YES, Grid, NSEW

from PIL import Image, ImageTk

# for multi-file projects see LICENSE file for authorship info
# for single file projects, insert following information
__author__ = 'Charlie Taylor'
__copyright__ = 'Copyright (c) 2016 Charlie Taylor'
__license__ = 'GPL-3'
exec( open(os.path.join( here,'_version.py' )).read() )  # creates local __version__ variable
__email__ = "cet@appliedpython.com"
__status__ = "3 - Alpha" # "3 - Alpha", "4 - Beta", "5 - Production/Stable"

from plot_area import PlotArea        

class Point( object ):
    def __init__(self, x, y):
        
        self.x = x
        self.y = y
        
    def get_str(self):
        return '%g, %g'%(self.x, self.y)
    
    def set_xy(self, x, y):
        self.x = x
        self.y = y
    
    def get_xy(self):
        return self.x, self.y

class DigiPlot(object):
    """A tkinter code that helps users digitize curves from plot images."""

    def __init__(self, master):
        
        self.initComplete = 0
        
        self.frame = Frame(master, width=555, height=546)
        self.frame.pack(fill=BOTH, expand=YES)
        self.master = master
        self.x, self.y, self.w, self.h = -1,-1,-1,-1

        # bind master to <Configure> in order to handle any resizing, etc.
        # postpone self.master.bind("<Configure>", self.Master_Configure)
        self.master.bind('<Enter>', self.bindConfigure)
        #self.master.bind('<Button-1>', self.bindConfigure)
        
        
        self.master.title("DigiPlot")
        
        # ======================  Make a Grid-Oriented Frame =============
        Grid.rowconfigure(master, 1, weight=1)
        Grid.columnconfigure(master, 1, weight=1)


        self.Defined_Points_Label = Label(self.frame,text="Defined Points", width="15")
        self.Defined_Points_Label.grid(row=0, column=0, in_=self.frame)

        # Listbox for points
        lbframe = Frame( self.frame )
        self.Defined_Points_Listbox_frame = lbframe
        scrollbar = Scrollbar(lbframe, orient=VERTICAL)
        self.Defined_Points_Listbox = Listbox(lbframe, width="35", selectmode="extended", 
                                              yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.Defined_Points_Listbox.yview)
        scrollbar.pack(side=RIGHT, fill=Y)
        self.Defined_Points_Listbox.pack(side=LEFT, fill=BOTH)#, expand=YES)
        self.Defined_Points_Listbox_frame.grid(row=1, column=0, sticky="ns", in_=self.frame)
        self.Defined_Points_Listbox.bind("<ButtonRelease-1>", self.Defined_Points_Listbox_Click)

        # Canvas for plot
        self.Plot_Canvas = Canvas(self.frame,width="400", height="400")
        self.Plot_Canvas.grid(row=1, column=1, sticky=NSEW, in_=self.frame)
        self.Plot_Canvas.bind("<ButtonRelease-1>", self.Plot_Canvas_Click)
        self.Plot_Canvas.bind("<Configure>", self.Plot_Canvas_Resize)



        # ======================  End of Grid-Oriented Frame =============
        # make a Status Bar
        self.statusMessage = StringVar()
        self.statusMessage.set("")
        self.statusbar = Label(self.master, textvariable=self.statusMessage, bd=1, relief=SUNKEN)
        self.statusbar.pack(anchor=SW, fill=X, side=BOTTOM)
        
        self.statusMessage.set("Welcome to temp")
        # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menuStructure"
        self.menuBar = Menu(self.master, relief = "raised", bd=2)

        top_File = Menu(self.menuBar, tearoff=0)

        top_File.add("command", label = "Import Image", command = self.menu_File_Import_Image)
        top_File.add("command", label = "Save CSV", command = self.menu_File_Save_CSV)
        self.menuBar.add("cascade", label="File", menu=top_File)

        top_Anchor_Plot = Menu(self.menuBar, tearoff=0)

        top_Anchor_Plot.add("command", label = "Set Xmin", command = self.menu_Anchor_Plot_Set_Xmin)
        top_Anchor_Plot.add("command", label = "Set Xmax", command = self.menu_Anchor_Plot_Set_Xmax)
        top_Anchor_Plot.add("command", label = "Set Ymin", command = self.menu_Anchor_Plot_Set_Ymin)
        top_Anchor_Plot.add("command", label = "Set Ymax", command = self.menu_Anchor_Plot_Set_Ymax)
        self.menuBar.add("cascade", label="Anchor Plot", menu=top_Anchor_Plot)
        self.menuBar.add("command", label = "Help", command = self.menu_Help)

        top_Exit = Menu(self.menuBar, tearoff=0)

        top_Exit.add("command", label = "Quit", command = self.menu_Exit_Quit)
        self.menuBar.add("cascade", label="Exit", menu=top_Exit)

        self.master.config(menu=self.menuBar)
        
        self.master.sizefrom(who='program')
        self.master.resizable(True, True) # Linux may not respect this

        self.Plot_Canvas.focus_set()
            
        self.PA = PlotArea(w_canv=400, h_canv=400) # will hold PlotArea() when it is opened
        self.pointL = []
            
        
        # give a few milliseconds before calling bindConfigure
        self.master.after(100, lambda: self.bindConfigure(None))

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "Master_Configure"
    def bindConfigure(self, event):
        if not self.initComplete:
            self.initComplete = 1
            #self.Plot_Canvas.config(width=600, height=500)
            self.master.bind("<Configure>", self.Master_Configure)
            self.plot_points()


    def Master_Configure(self, event):
        
        if event.widget != self.master:
            if self.w > -1:
                return
        self.master.unbind('<Enter>')
            
        x = int(self.master.winfo_x())
        y = int(self.master.winfo_y())
        w = int(self.master.winfo_width())
        h = int(self.master.winfo_height())
        
        if (self.x, self.y, self.w, self.h) == (-1,-1,-1,-1):
            self.x, self.y, self.w, self.h = x,y,w,h
            self.lb_init_width  = int(self.Defined_Points_Listbox_frame.winfo_width() )
            self.lb_init_height = int(self.Defined_Points_Listbox_frame.winfo_height())
            self.w_init = w
            self.h_init = h
            self.w_canv_init = int( self.Plot_Canvas.winfo_width() )

        if self.w!=w or self.h!=h:
            print( "Master reconfigured... make resize adjustments")
            self.w=w
            self.h=h
        
        print('self.frame.w =', self.frame.winfo_width())
        self.frame.update()
        self.frame.update_idletasks()
        
        #dw = max(100, self.w - self.w_init + self.w_canv_init - 1)
        #dh = max(100, self.h - self.h_init + self.lb_init_height)
        
        #self.Plot_Canvas.config(width=dw, height=dh)
        
        #self.PA.set_canvas_wh(dw, dh)
        #self.plot_points()

    def Plot_Canvas_Resize(self, event):
        self.PA.set_canvas_wh(event.width, event.height)
        self.plot_points()
        

    def plot_points(self):
        s = 3
        s2 = s+3
        s3 = s+5
        
        # make coordinate axes
        self.Plot_Canvas.delete("all")
        
        self.photo_image = self.PA.get_tk_photoimage(greyscale=False)
        self.Plot_Canvas.create_image(0,0, anchor=NW, image=self.photo_image )
                                       
        img_slice_resized = self.PA.get_zoomed_offset_img(greyscale=True)
        #print('img_slice_resized.size=',img_slice_resized.size)
                    
        iL = [i for i in self.Defined_Points_Listbox.curselection()]
        if (len(iL)>0):
            isel = iL[0]
        else:
            isel = -1

        # Show points
        for ip,P in enumerate(self.pointL):
            x,y = P.get_xy()
            i,j = self.PA.get_ij_at_xy( x,y )
            
            if min(i,j)>=0:
                if (isel>=0) and (ip==isel):
                    self.Plot_Canvas.create_rectangle((i-s2, j-s2, i+s2, j+s2), outline="black", fill="orange")
                else:
                    self.Plot_Canvas.create_rectangle((i-s2, j-s2, i+s2, j+s2), outline="blue", fill="cyan")
                
    
    def add_point(self, x_float, y_float):
        
        itemL = list(self.Defined_Points_Listbox.get(0, END))
        iL = [i for i in self.Defined_Points_Listbox.curselection()]
        
        if len(itemL)==0:
            self.pointL.append( Point(x_float, y_float) )
            self.Defined_Points_Listbox.insert(END, self.pointL[-1].get_str() )
            self.select_pointL(0)
        else:
            
            if len(iL)==0:
                self.pointL.append( Point(x_float, y_float) )
                self.Defined_Points_Listbox.insert(END, self.pointL[-1].get_str() )
                self.select_pointL(END)
            else:
                self.pointL.insert(iL[0]+1, Point(x_float, y_float) )
                self.Defined_Points_Listbox.insert(iL[0]+1, self.pointL[iL[0]+1].get_str() )
                self.select_pointL(iL[0]+1)
                
                #print( 'self.pointL =',self.pointL)
                
        self.statusMessage.set("added point x,y=%g, %g"%(x_float, y_float))
                
    def select_pointL(self, i ):
        
        self.Defined_Points_Listbox.selection_clear(0, END)
        if i==END:
            self.Defined_Points_Listbox.select_set( END )
            i = len(self.pointL)-1
        else:
            self.Defined_Points_Listbox.select_set( i )
            
        self.plot_points()

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_File_Import_Image"
    def menu_File_Import_Image(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_File_Import_Image"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_File_Import_Image")
        print( "called menu_File_Import_Image")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_File_Save_CSV"
    def menu_File_Save_CSV(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_File_Save_CSV"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_File_Save_CSV")
        print( "called menu_File_Save_CSV")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Xmin"
    def menu_Anchor_Plot_Set_Xmin(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Xmin"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Xmin")
        print( "called menu_Anchor_Plot_Set_Xmin")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Xmax"
    def menu_Anchor_Plot_Set_Xmax(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Xmax"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Xmax")
        print( "called menu_Anchor_Plot_Set_Xmax")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Ymin"
    def menu_Anchor_Plot_Set_Ymin(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Ymin"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Ymin")
        print( "called menu_Anchor_Plot_Set_Ymin")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Ymax"
    def menu_Anchor_Plot_Set_Ymax(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Ymax"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Ymax")
        print( "called menu_Anchor_Plot_Set_Ymax")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Help"
    def menu_Help(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Help"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Help")
        print( "called menu_Help")


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Exit_Quit"
    def menu_Exit_Quit(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Exit_Quit"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Exit_Quit")
        print( "called menu_Exit_Quit")
        self.quit()

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=1"
    def Plot_Canvas_Click(self, event): #click method for component ID=1
        pass
        # >>>>>>insert any user code below this comment for section "compID=1"
        # replace, delete, or comment-out the following
        
        self.plot_points()
        
        #print( "executed method Plot_Canvas_Click")
        #self.statusMessage.set("executed method Plot_Canvas_Click")
        #print( "clicked in canvas at i,j =",event.x,event.y)
        
        x,y = self.PA.get_xy_at_ij(event.x, event.y)
        #print('x=%g, y=%g'%(x,y))
        self.add_point(x,y)

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=2"
    def Defined_Points_Listbox_Click(self, event): #click method for component ID=2
        pass
        # >>>>>>insert any user code below this comment for section "compID=2"
        # replace, delete, or comment-out the following
        print( "executed method Defined_Points_Listbox_Click")
        self.statusMessage.set("executed method Defined_Points_Listbox_Click")
        print( "current selection(s) =",self.Defined_Points_Listbox.curselection())
        labelL = []
        for i in self.Defined_Points_Listbox.curselection():
            labelL.append( self.Defined_Points_Listbox.get(i))
        print( "current label(s) =",labelL)
        # use self.Defined_Points_Listbox.insert(0, "item zero")
        #     self.Defined_Points_Listbox.insert(index, "item i")
        #            OR
        #     self.Defined_Points_Listbox.insert(END, "item end")
        #   to insert items into the list box

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "standard_message_dialogs"

    # standard message dialogs... showinfo, showwarning, showerror
    def ShowInfo(self, title='Title', message='your message here.'):
        tkMessageBox.showinfo( title, message )
        return
    def ShowWarning(self, title='Title', message='your message here.'):
        tkMessageBox.showwarning( title, message )
        return
    def ShowError(self, title='Title', message='your message here.'):
        tkMessageBox.showerror( title, message )
        return
        
    # standard question dialogs... askquestion, askokcancel, askyesno, or askretrycancel
    # return True for OK, Yes, Retry, False for Cancel or No
    def AskYesNo(self, title='Title', message='your question here.'):
        return tkMessageBox.askyesno( title, message )
    def AskOK_Cancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askokcancel( title, message )
    def AskRetryCancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askretrycancel( title, message )
        
    # return "yes" for Yes, "no" for No
    def AskQuestion(self, title='Title', message='your question here.'):
        return tkMessageBox.askquestion( title, message )
    # END of standard message dialogs

    # >>>>>>insert any user code below this comment for section "standard_message_dialogs"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "standard_file_dialogs"
    # standard file dialogs... askdirectory, askopenfile, asksaveasfilename

    # return a string containing directory name
    def AskDirectory(self, title='Choose Directory', initialdir="."):
        dirname = tkFileDialog.askdirectory(parent=self.master,initialdir=initialdir,title=title)
        return dirname # <-- string
        
    # return an OPEN file type object OR None (opened using mode, 'r','rb','w','wb')
    # WARNING... opening file with mode 'w' or 'wb' will erase contents
    def AskOpenFile(self, title='Choose File', mode='rb', initialdir='.', filetypes=None):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]
        fileobj = tkFileDialog.askopenfile(parent=self.master,mode=mode,title=title,
            initialdir=initialdir, filetypes=filetypes)
        
        # if opened, then fileobj.name contains the name string
        return fileobj # <-- an opened file, or the value None
        
    # return a string containing file name (the calling routine will need to open the file)
    def AskSaveasFilename(self, title='Save File', filetypes=None, initialfile=''):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]

        fileName = tkFileDialog.asksaveasfilename(parent=self.master,filetypes=filetypes, initialfile=initialfile ,title=title)
        return fileName # <-- string
        
    # END of standard file dialogs

    # >>>>>>insert any user code below this comment for section "standard_file_dialogs"


# tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "end"

def main():
    root = Tk()
    app = DigiPlot(root)
    root.mainloop()

if __name__ == '__main__':
    main()



import unittest
# import unittest2 as unittest # for versions of python < 2.7

"""
        Method                  Checks that
self.assertEqual(a, b)           a == b   
self.assertNotEqual(a, b)        a != b   
self.assertTrue(x)               bool(x) is True  
self.assertFalse(x)              bool(x) is False     
self.assertIs(a, b)              a is b
self.assertIsNot(a, b)           a is not b
self.assertIsNone(x)             x is None 
self.assertIsNotNone(x)          x is not None 
self.assertIn(a, b)              a in b
self.assertNotIn(a, b)           a not in b
self.assertIsInstance(a, b)      isinstance(a, b)  
self.assertNotIsInstance(a, b)   not isinstance(a, b)  

See:
      https://docs.python.org/2/library/unittest.html
         or
      https://docs.python.org/dev/library/unittest.html
for more assert options
"""

import sys, os

here = os.path.abspath(os.path.dirname(__file__)) # Needed for py.test
up_one = os.path.split( here )[0]  # Needed to find digiplot development version
if here not in sys.path[:2]:
    sys.path.insert(0, here)
if up_one not in sys.path[:2]:
    sys.path.insert(0, up_one)

from digiplot.plot_image import PlotImage

class MyTest(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.myclass = PlotImage(1000, 1000)

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        del( self.myclass )

    def test_should_always_pass_cleanly(self):
        """Should always pass cleanly."""
        pass

    def test_myclass_existence(self):
        C = self.myclass
        self.assertTrue(C)
        self.assertEqual(C.i_origin, 100)
        self.assertEqual(C.j_origin, 900)

    def test_set_origin_xy(self):
        C = self.myclass
        C.set_origin_xy( 10., 1000. )
        self.assertEqual( (C.x_origin,C.y_origin), (10., 1000.))

    def test_set_ix_origin(self):
        C = self.myclass
        C.set_ix_origin(10, 99.9)
        self.assertEqual( (C.i_origin,C.x_origin), (10, 99.9))

    def test_set_jy_origin(self):
        C = self.myclass
        C.set_jy_origin(10, 99.9)
        self.assertEqual( (C.j_origin,C.y_origin), (10, 99.9))

    def test_define_origin_ij(self):
        C = self.myclass
        C.define_origin_ij(500, 500)
        self.assertEqual( (C.i_origin,C.j_origin), (500, 500))

    def test_set_imax_xmax(self):
        C = self.myclass
        C.set_imax_xmax(222, 222.)
        self.assertEqual( (C.imax,C.xmax), (222, 222.0))

    def test_set_jmax_ymax(self):
        C = self.myclass
        C.set_jmax_ymax(222, 222.)
        self.assertEqual( (C.jmax,C.ymax), (222, 222.0))

    def test_get_xy_at_ij(self):
        x,y = self.myclass.get_xy_at_ij(900, 100)
        
        self.assertEqual((x,y), (1.,1.))

    def test_get_a_few_xy_at_ij(self):
        x,y = self.myclass.get_xy_at_ij(800, 200)
        self.assertEqual((x,y), (0.875, 0.875))

        x,y = self.myclass.get_xy_at_ij(500, 500)
        self.assertEqual((x,y), (0.5, 0.5))


if __name__ == '__main__':
    # Can test just this file from command prompt
    #  or it can be part of test discovery from nose, unittest, pytest, etc.
    unittest.main()





class PlotImage(object):
    
    """Logic for interpreting a plot image."""
    def __init__(self, w=1000, h=1000):
        self.w = w
        self.h = h

        self.i_origin = int(w * 0.1) # lower left
        self.j_origin = int(h * 0.9) # lower left
        self.x_origin = 0.0
        self.y_origin = 0.0

        self.imax = int(w * 0.9)
        self.jmax = int(h * 0.1)
        self.xmax = 1.0
        self.ymax = 1.0

    def define_origin_ij(self, i, j):
        """Place the origin of the 2D plot at image coordinates i,j."""
        self.i_origin = i
        self.j_origin = j
        
    def set_origin_xy(self, x, y):
        self.x_origin = x 
        self.y_origin = y
    
    def set_ix_origin(self, i, x):
        self.i_origin = i
        self.x_origin = x
    
    def set_jy_origin(self, j, y):
        self.j_origin = j
        self.y_origin = y
    
    def set_imax_xmax(self, imax, xmax):
        self.imax = imax
        self.xmax = xmax
    
    def set_jmax_ymax(self, jmax, ymax):
        self.jmax = jmax
        self.ymax = ymax
        
    def get_xy_at_ij(self, i,j):
        di = i - self.i_origin
        dj = self.j_origin - j # note LL vs UL
        
        dx = self.xmax - self.x_origin
        dy = self.ymax - self.y_origin
        
        x = self.x_origin + dx * di / (self.imax - self.i_origin)
        y = self.y_origin + dy * dj / (self.j_origin - self.jmax) # note LL vs UL
        
        return x,y

        
    def get_canvas_i(self, x_float):
        fx = (x_float-self.x_origin) / (self.xmax-self.x_origin)
        if fx>=0.0 and fx<=1.0:
            i = int(fx * self.w + 0.5)
            return i
        else:
            return -1 # if not on canvas
        
    def get_canvas_j(self, y_float):
        fy = (y_float-self.y_origin) / (self.ymax-self.y_origin)
        if fy>=0.0 and fy<=1.0:
            j = self.h - int(fy * self.h + 0.5)
            return j
        else:
            return -1 # if not on canvas

    def get_ij_at_xy(self, x,y):
        
        i = self.get_canvas_i( x_float )
        j = self.get_canvas_j( y_float )
        if i>=0 and j>=0:
            return i,j
        else:
            return -1, -1 # if not on canvas




import os, sys
import base64
import fnmatch
import mimetypes

def build_py_resource( source_dir, py_out_prefix='myqt', glob_desc='*' ):
    
    parent_dir, res_dir = os.path.split( source_dir )
    print 'parent_dir =',parent_dir
    print '  res_dir  =',res_dir

    py_out_files = '%s_%s_resource_files.py'%(py_out_prefix, res_dir)
    pyOut_files = open(py_out_files,'w')
    pyOut_files.write('''

# parent_dir = %s
target_dir = r"%s"

file_typeD = {} # file type description. index=resource path, value=mime type
'''%(parent_dir, res_dir))
    
    py_out_file_data = '%s_%s_resources.py'%(py_out_prefix, res_dir)
    pyOut_file_data = open(py_out_file_data,'w')

    pyOut_file_data.write('''

import base64

# parent_dir = %s
target_dir = r"%s"

file_dataD = {} # file data dict. index=resource path, value=file contents

'''%(parent_dir, res_dir))
    
    fileL = rec_glob(source_dir, glob_desc)
    for fname in fileL:
        mime = mimetypes.guess_type(fname)
        rel_path = os.path.relpath( fname, source_dir )
        resrc_path = rel_path # os.path.join( res_dir, rel_path )
        print mime, resrc_path
        
        image_file = open(fname, 'rb')
        encoded_string = base64.b64encode(image_file.read())
        image_file.close()

        pyOut_files.write( 'file_typeD[r"%s"]="%s"\n'%(resrc_path,mime))
        pyOut_file_data.write( '\n\nfile_dataD[r"%s"]="""\n'%resrc_path)
        i = 0
        while i < len(encoded_string):
            pyOut_file_data.write( encoded_string[i:i+76] + '\n' )
            i += 76
        pyOut_file_data.write( '"""\n\n' )
        
        
    pyOut_file_data.close()
    pyOut_files.close()
        


def rec_glob_get_dirs(path):
    d=[]
    try:
        for i in os.listdir(path):
            if os.path.isdir(path+i):
                d.append(os.path.basename(i))
    except:pass
    return d

def rec_glob(path,mask):
    l=[]
    if path[-1]!='\\':
        path=path+'\\'
    for i in rec_glob_get_dirs(path):
        l=l+rec_glob(path+i,mask)
    try:
        for i in os.listdir(path):
            ii=i
            i=path+i
            if os.path.isfile(i):
                if fnmatch.fnmatch(ii,mask):
                    l.append(i)
    except:pass
    return l


if __name__ == '__main__':
    
    source_dir = r'D:\py_proj_2017\DigiPlot\digiplot\backup'
    build_py_resource( source_dir, py_out_prefix='digiplot', glob_desc='samp*.png' )
    
    
    


#!/usr/bin/env python
# -*- coding: ascii -*-
from __future__ import absolute_import
from __future__ import print_function

r"""
A tkinter code that helps users digitize curves from plot images.

If a 2D plot is available only on paper or an image file, but the user needs the x,y data pairs in digital form, DigiPlot will help to create those x,y data pairs.


DigiPlot
Copyright (C) 2017  Charlie Taylor

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

-----------------------

"""
import os
here = os.path.abspath(os.path.dirname(__file__))
import sys
if sys.version_info < (3,):
    from future import standard_library
    standard_library.install_aliases()
    import tkFileDialog
    import tkMessageBox
else:
    import tkinter.filedialog as tkFileDialog
    import tkinter.messagebox as tkMessageBox

#  Need to redefine object here
# pylint: disable=W0622, W0703, W0612, W0403, C0330
from builtins import object

import tkinter.font
#from tkinter import *
from tkinter import Menu, StringVar, Label, SUNKEN, SW, X, BOTTOM, Frame, NE, NW,\
    BOTH, TOP, Button, W, LEFT, SE, Scrollbar, VERTICAL, Text, RIGHT, Y, END, Tk,\
    Canvas, Listbox, Entry

from PIL import Image, ImageTk

# for multi-file projects see LICENSE file for authorship info
# for single file projects, insert following information
__author__ = 'Charlie Taylor'
__copyright__ = 'Copyright (c) 2016 Charlie Taylor'
__license__ = 'GPL-3'
exec( open(os.path.join( here,'_version.py' )).read() )  # creates local __version__ variable
__email__ = "cet@appliedpython.com"
__status__ = "3 - Alpha" # "3 - Alpha", "4 - Beta", "5 - Production/Stable"

from plot_area import PlotArea        

class DigiPlot(object):
    """A tkinter code that helps users digitize curves from plot images."""

    def __init__(self, master):
        self.initComplete = 0
        frame = Frame(master, width=1020, height=760)
        self.frame = frame
        frame.pack()
        self.master = master
        self.x, self.y, self.w, self.h = -1,-1,-1,-1
        
        # bind master to <Configure> in order to handle any resizing, etc.
        # postpone self.master.bind("<Configure>", self.Master_Configure)
        self.master.bind('<Enter>', self.bindConfigure)
        

        self.master.title("DigiPlot")


        self.Xmax_Entry = Entry(self.master,width="15")
        self.Xmax_Entry.place(x=64, y=52, width=96, height=21)
        self.Xmax_Entry_StringVar = StringVar()
        self.Xmax_Entry.configure(textvariable=self.Xmax_Entry_StringVar)
        self.Xmax_Entry_StringVar.set("10")
        self.Xmax_Entry_StringVar_traceName = self.Xmax_Entry_StringVar.trace_variable("w", self.Xmax_Entry_StringVar_Callback)

        self.Xmin_Entry = Entry(self.master,width="15")
        self.Xmin_Entry.place(x=64, y=12, width=96, height=21)
        self.Xmin_Entry_StringVar = StringVar()
        self.Xmin_Entry.configure(textvariable=self.Xmin_Entry_StringVar)
        self.Xmin_Entry_StringVar.set("-10")
        self.Xmin_Entry_StringVar_traceName = self.Xmin_Entry_StringVar.trace_variable("w", self.Xmin_Entry_StringVar_Callback)

        self.Ymax_Entry = Entry(self.master,width="15")
        self.Ymax_Entry.place(x=64, y=132, width=95, height=20)
        self.Ymax_Entry_StringVar = StringVar()
        self.Ymax_Entry.configure(textvariable=self.Ymax_Entry_StringVar)
        self.Ymax_Entry_StringVar.set("10")
        self.Ymax_Entry_StringVar_traceName = self.Ymax_Entry_StringVar.trace_variable("w", self.Ymax_Entry_StringVar_Callback)

        self.Ymin_Entry = Entry(self.master,width="15")
        self.Ymin_Entry.place(x=64, y=92, width=95, height=20)
        self.Ymin_Entry_StringVar = StringVar()
        self.Ymin_Entry.configure(textvariable=self.Ymin_Entry_StringVar)
        self.Ymin_Entry_StringVar.set("-10")
        self.Ymin_Entry_StringVar_traceName = self.Ymin_Entry_StringVar.trace_variable("w", self.Ymin_Entry_StringVar_Callback)

        self.Label_1 = Label(self.master,text="Xmin", width="15")
        self.Label_1.place(x=12, y=12, width=30, height=22)

        self.Label_2 = Label(self.master,text="Xmax", width="15")
        self.Label_2.place(x=12, y=52, width=30, height=22)

        self.Label_3 = Label(self.master,text="Ymin", width="15")
        self.Label_3.place(x=12, y=92, width=30, height=22)

        self.Label_4 = Label(self.master,text="Ymax", image="", width="15")
        self.Label_4.place(x=12, y=132, width=30, height=22)



        self.Canvas_1 = Canvas(self.master, width="764", height="720")
        self.Canvas_1.place(x=240, y=12)
        self.Canvas_1.bind("<Button-1>", self.Canvas_1_Click)
        
        self.Canvas_1.bind("<Motion>", self.Canvas_Hover)
        self.Canvas_1.bind("<Button-3>", self.Canvas_Find_Closest)
        self.Canvas_1.bind("<B3-Motion>", self.Canvas_Drag_Axes)


        lbframe = Frame( self.master )
        self.Listbox_1_frame = lbframe
        scrollbar = Scrollbar(lbframe, orient=VERTICAL)
        self.Listbox_1 = Listbox(lbframe, width="15", selectmode="single", 
                                 yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.Listbox_1.yview)
        scrollbar.pack(side=RIGHT, fill=Y)
        self.Listbox_1.pack(side=LEFT, fill=BOTH, expand=1)

        self.Listbox_1_frame.place(x=12, y=432, width=160, height=250)
        self.Listbox_1.bind("<ButtonRelease-1>", self.Listbox_1_Click)
        self.Listbox_1.bind("<Double-Button-1>", self.Listbox_1_DoubleClick)


        # make a Status Bar
        self.statusMessage = StringVar()
        self.statusMessage.set("")
        self.statusbar = Label(self.master, textvariable=self.statusMessage, bd=1, relief=SUNKEN)
        self.statusbar.pack(anchor=SW, fill=X, side=BOTTOM)
        self.menuBar = Menu(master, relief = "raised", bd=2)

        top_File = Menu(self.menuBar, tearoff=0)

        top_File.add("command", label = "Open", command = self.menu_Img_File_Open)
        top_File.add("command", label = "Save", command = self.menu_File_Save)
        self.menuBar.add("cascade", label="File", menu=top_File)

        self.menuBar.add_command(label='Help', command=self.help_popup)

        master.config(menu=self.menuBar)
        master.bind('<Escape>', self.Clear_Pending_Actions)


        
        self.w_canv = int(self.Canvas_1.cget("width"))
        self.h_canv = int(self.Canvas_1.cget("height"))
        
        self.Canvas_1.config( bg='#ddffff' )
        self.canvas_bg_img = None # Use Background/Import menu
        self.canvas_bg_photo = None # Needed by Canvas object (ImageTk.PhotoImage)
        self.canvas_bg_img_is_greyscale = False
        self.canvas_bg_img_x_offset = 0
        self.canvas_bg_img_y_offset = 0


        # =========================== 
        self.PA = PlotArea(w_canv=self.w_canv, h_canv=self.h_canv) # will hold PlotArea() when it is opened
        self.pointL = []

        
        self.last_right_click_pos = (0,0) # any action calling Canvas_Find_Closest will set
        self.last_right_click_bbox = (self.PA.x_origin, self.PA.xmax, self.PA.y_origin, self.PA.ymax)
        
        self.pathopen = ''
        self.fopen = ''
        self.dirname = ''
        if len( sys.argv ) == 2:
            fName = sys.argv[1]
            if fName.find('.')<0:
                fName += '.png'
            self.pathopen = os.path.abspath(fName)
            if os.path.isfile( self.pathopen ): # if file exists, read it as a definition file
                
                self.canvas_bg_img_is_greyscale = False
                
                self.PA.open_img_file( self.pathopen )
                
                self.canvas_bg_photo = self.PA.get_tk_photoimage(greyscale=self.canvas_bg_img_is_greyscale)
                self.plot_points()
            else:
                MainWin.statusMessage.set('file "%s" does not exist'%fName)
        
        

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Img_File_Open"
    def menu_Img_File_Open(self):
        self.statusMessage.set("called menu_Img_File_Open")
        print( "called menu_Img_File_Open")
        
        self.statusMessage.set("called menu_Background_Import_PNG")
        
        filetypes = [
            ('PNG Image','*.png'),
            ('Any File','*.*')]
        img_path = tkFileDialog.askopenfilename(parent=self.master,title='Open Image file', 
            filetypes=filetypes)
            
        if img_path:
            self.canvas_bg_img_is_greyscale = False
            
            self.PA.open_img_file( img_path )
            if self.PA.img is not None:
                self.canvas_bg_photo = self.PA.get_tk_photoimage(greyscale=self.canvas_bg_img_is_greyscale)
            self.plot_points()

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_File_Save"
    def menu_File_Save(self):
        self.statusMessage.set("called menu_File_Save")
        print( "called menu_File_Save")
        
        if self.pathopen:
            name = self.fopen
        else:
            name = '*.csv'
        self.saveFile(name)
        
    def saveFile(self, fileDesc='*.csv'):
        print( 'Save File')
        fsave = tkFileDialog.asksaveasfilename(parent=self.master, title='Saving Perimeter or Path file', 
            initialfile=fileDesc)
        
        if fsave:
            if fsave.find('.')<0:
                fsave += '.csv'
            
            
            self.pathopen = fsave
            self.set_file_properties()
            print( 'Saving to:',self.fopen)
            
            self.master.title("tk_perimeter: %s"%self.fopen)
            
            fOut = open( self.pathopen, 'w' )
            
            for P in self.pointL:
                fOut.write( '%s\n'%P.get_str() )
                #print 'Saving Line:',P.get_str()
            fOut.close()

        
    
    def get_floats_from_event_xy(self, event):
        if hasattr(self, 'PA'):
            return self.PA.get_xy_at_ij( event.x, event.y )
        else:
            return None, None
        
        
    def get_canvas_ij(self, x_float, y_float):
        return self.PA.get_ij_at_xy( x_float, y_float )

    def Clear_Pending_Actions(self, event):
        self.statusMessage.set("Cleared pending actions.")
        
        self.is_dragging = False
        
        self.is_dragging_all = False
        self.is_dragging_all_start_posx = 0.0
        self.is_dragging_all_start_posy = 0.0
        
        self.Canvas_1.bind("<Button-1>", self.Canvas_1_Click)
    def bindConfigure(self, event):
        if not self.initComplete:
            self.master.bind("<Configure>", self.Master_Configure)
            self.initComplete = 1


    def Master_Configure(self, event):
        if event.widget != self.master:
            if self.w != -1:
                return
        x = int(self.master.winfo_x())
        y = int(self.master.winfo_y())
        w = int(self.master.winfo_width())
        h = int(self.master.winfo_height())
        if (self.x, self.y, self.w, self.h) == (-1,-1,-1,-1):
            self.x, self.y, self.w, self.h = x,y,w,h


        if self.w!=w or self.h!=h:
            print( "Master reconfigured... make resize adjustments")
            self.w=w
            self.h=h
            
            self.Canvas_1.config( width=w-250, height=h-42 )

        self.w_canv = int(self.Canvas_1.cget("width"))
        self.h_canv = int(self.Canvas_1.cget("height"))
        
        if self.PA.img is not None:
            self.PA.reset_canvas_wh(self.w_canv, self.h_canv)
            self.canvas_bg_photo = self.PA.get_tk_photoimage(greyscale=self.canvas_bg_img_is_greyscale)
            self.plot_points()
        

    def Canvas_Find_Closest(self, event): #click method for component ID=1
        ix = int(event.x)
        iy = int(event.y)   #self.h_canv - int(event.y)
        #print "Right Click event ix,iy=",ix,iy
        
        self.last_right_click_pos = (ix, iy)
        self.last_right_click_bbox = (self.PA.x_origin, self.PA.xmax, self.PA.y_origin, self.PA.ymax)
        
        #dx = max(1.0E-6, abs(self.xmax - self.xmin))
        #dy = max(1.0E-6, abs(self.ymax - self.ymin))
        
        # Find the closest point
        ibest = 0
        dbest = 1.0E12
        for i,P in enumerate( self.pointL ):
            x,y = P.get_xy()
            ii,jj = self.get_canvas_ij(x,y)
            d = (ix-ii)**2 + (iy-jj)**2
            if d<dbest:
                ibest = i
                dbest = d
            
                
        self.select_pointL( ibest )
        
        print( 'ibest=%s'%(ibest, ))
        return ibest


    def Canvas_Hover(self, event):
        x = int(event.x)
        y = self.h_canv - int(event.y)
        
        x_move, y_move = self.get_floats_from_event_xy(event)
        
        try:
            x_str = '%g'%x_move
            y_str = '%g'%y_move
        except:
            x_str = '%s'%x_move
            y_str = '%s'%y_move
            
        
        self.statusMessage.set(x_str + ', ' + y_str)

    def Canvas_Drag_Axes(self, event):
        
        di = event.x - self.last_right_click_pos[0]
        dj = event.y - self.last_right_click_pos[1]
        #print 'Right Drag x,y =',event.x, event.y,'   di,dj=',di,dj
        
        xmin, xmax, ymin, ymax = self.last_right_click_bbox


        dx = -(xmax - xmin) * di / float(self.w_canv)
        dy = (ymax - ymin) * dj / float(self.h_canv)

        xlo = self.get_snap_value( xmin + dx )
        xhi = self.get_snap_value( xmax + dx )

        ylo = self.get_snap_value( ymin + dy )
        yhi = self.get_snap_value( ymax + dy )

        self.Xmin_Entry_StringVar.set( '%g'%xlo )
        self.Xmax_Entry_StringVar.set( '%g'%xhi )
        self.Ymin_Entry_StringVar.set( '%g'%ylo )
        self.Ymax_Entry_StringVar.set( '%g'%yhi )
        
        self.plot_points()

    def Listbox_1_DoubleClick(self, event):
        
        self.EditXYR_Button_Click( event )

        
    def Canvas_1_Click(self, event): #click method for component ID=1
        self.statusMessage.set("executed method Canvas_1_Click")
        x = int(event.x)
        y = self.h_canv - int(event.y)
        x_float, y_float = self.get_floats_from_event_xy(event)
        
        if x_float is not None:
            self.add_point( x_float, y_float)
            
            self.plot_points()
                
    def select_pointL(self, i ):
        
        self.Listbox_1.selection_clear(0, END)
        if i==END:
            self.Listbox_1.select_set( END )
            i = len(self.pointL)-1
        else:
            self.Listbox_1.select_set( i )
            
        self.plot_points()

    def plot_points(self):
        
            
        s = 3
        s2 = s+3
        s3 = s+5
        
        # make coordinate axes
        self.Canvas_1.delete("all")
        
        if self.PA.img is not None:
            self.Canvas_1.create_image(self.canvas_bg_img_x_offset, self.canvas_bg_img_y_offset, 
                                       anchor=NW, image=self.canvas_bg_photo)
                                       
                    
        iL = [i for i in self.Listbox_1.curselection()]
        if (len(iL)>0):
            isel = iL[0]
        else:
            isel = -1

        # Show points
        for ip,P in enumerate(self.pointL):
            x,y = P.get_xy()
            i,j = self.get_canvas_ij( x,y )
            
            if min(i,j)>=0:
                if (isel>=0) and (ip==isel):
                    self.Canvas_1.create_rectangle((i-s2, j-s2, i+s2, j+s2), outline="black", fill="orange")
                else:
                    self.Canvas_1.create_rectangle((i-s, j-s, i+s, j+s), outline="blue", fill="blue")
                
        
    
    def add_point(self, x_float, y_float):
        
        itemL = list(self.Listbox_1.get(0, END))
        iL = [i for i in self.Listbox_1.curselection()]
        
        if len(itemL)==0:
            self.pointL.append( Point(x_float, y_float) )
            self.Listbox_1.insert(END, self.pointL[-1].get_str() )
            #self.Listbox_1.select_set(0)
            self.select_pointL(0)
        else:
            
            if len(iL)==0:
                self.pointL.append( Point(x_float, y_float) )
                self.Listbox_1.insert(END, self.pointL[-1].get_str() )
                #self.Listbox_1.select_set(END)
                self.select_pointL(END)
            else:
                self.pointL.insert(iL[0]+1, Point(x_float, y_float) )
                self.Listbox_1.insert(iL[0]+1, self.pointL[iL[0]+1].get_str() )
                #self.Listbox_1.select_set(iL[0]+1)
                self.select_pointL(iL[0]+1)
                
                #print 'self.pointL =',self.pointL
        
        
    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=15"
    def Listbox_1_Click(self, event): #click method for component ID=15
        #print "executed method Listbox_1_Click"
        self.statusMessage.set("executed method Listbox_1_Click")
        
        iL = [i for i in self.Listbox_1.curselection()]
        if (len(iL)>0):
            self.select_pointL(iL[0])

    def RePlot_Button_Click(self, event):
        self.plot_points()
        

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=16"
    def Del_Button_Click(self, event): #click method for component ID=16
        #print "executed method Del_Button_Click"
        self.statusMessage.set("executed method Del_Button_Click")
        
        iL = [i for i in self.Listbox_1.curselection()]
        if len(iL)>0:
            #print 'Deleting item %i'%iL[0]
            
            self.Listbox_1.delete( iL[0] )
            self.pointL.pop( iL[0] )
            
            isel = iL[0]-1
            if isel < 0:
                isel = 0
            if len(self.pointL)>0:
                self.select_pointL(isel)
            

        self.plot_points()

    def MouseWheelHandler(self, event):

        if event.num == 5 or event.delta < 0:
            result = -1 
            self.MakeBigger_Button_Click(event)
        else:
            result = 1 
            self.MakeSmaller_Button_Click(event)

        #print 'MouseWheelHandler result =',result

    def MakeBigger_Button_Click(self, event):
        dx = (self.xmax - self.xmin) * 0.1/ 0.8
        xlo = self.xmin - dx
        xhi = self.xmax + dx

        dy = (self.ymax - self.ymin) * 0.1/ 0.8
        ylo = self.ymin - dy
        yhi = self.ymax + dy

        self.Xmin_Entry_StringVar.set( '%g'%xlo )
        self.Xmax_Entry_StringVar.set( '%g'%xhi )

        self.Ymin_Entry_StringVar.set( '%g'%ylo )
        self.Ymax_Entry_StringVar.set( '%g'%yhi )
        
        self.plot_points()
        
    def MakeSmaller_Button_Click(self, event):
        dx = (self.xmax - self.xmin) * 0.1
        xlo = self.xmin + dx
        xhi = self.xmax - dx

        dy = (self.ymax - self.ymin) * 0.1
        ylo = self.ymin + dy
        yhi = self.ymax - dy

        self.Xmin_Entry_StringVar.set( '%g'%xlo )
        self.Xmax_Entry_StringVar.set( '%g'%xhi )
        self.Ymin_Entry_StringVar.set( '%g'%ylo )
        self.Ymax_Entry_StringVar.set( '%g'%yhi )
        
        self.plot_points()
        
    def MakeFit_Button_Click(self, event):
        if len(self.pointL)>0:
            ptL = [pt.get_xy() for pt in self.pointL]
            
            xL = [p[0] for p in ptL]
            yL = [p[1] for p in ptL]
            
            xmin = min(xL)
            xmax = max(xL)
            ymin = min(yL)
            ymax = max(yL)
            
            dx = xmax - xmin
            dy = ymax - ymin
            
            xlo = xmin - dx/10.0
            xhi = xmax + dx/10.0

            ylo = ymin - dy/10.0
            yhi = ymax + dy/10.0
        
            self.Xmin_Entry_StringVar.set( '%g'%xlo )
            self.Xmax_Entry_StringVar.set( '%g'%xhi )
            self.Ymin_Entry_StringVar.set( '%g'%ylo )
            self.Ymax_Entry_StringVar.set( '%g'%yhi )
            
            self.plot_points()

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "Xmax_Entry_StringVar_Callback"
    def Xmax_Entry_StringVar_Callback(self, varName, index, mode):
        #print "Xmax_Entry_StringVar_Callback varName, index, mode",varName, index, mode
        self.statusMessage.set("    Xmax_Entry_StringVar = "+self.Xmax_Entry_StringVar.get())
        #print "    new StringVar value =",self.Xmax_Entry_StringVar.get()
        
        try:
            self.xmax = float( self.Xmax_Entry_StringVar.get() )
        except:
            pass
            
        self.PA.set_x_max( self.xmax )
        self.plot_points()


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "Xmin_Entry_StringVar_Callback"
    def Xmin_Entry_StringVar_Callback(self, varName, index, mode):
        #print "Xmin_Entry_StringVar_Callback varName, index, mode",varName, index, mode
        self.statusMessage.set("    Xmin_Entry_StringVar = "+self.Xmin_Entry_StringVar.get())
        #print "    new StringVar value =",self.Xmin_Entry_StringVar.get()
        
        try:
            self.xmin = float( self.Xmin_Entry_StringVar.get() )
        except:
            pass
        self.PA.set_x_origin( self.xmin )
        self.plot_points()


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "Ymax_Entry_StringVar_Callback"
    def Ymax_Entry_StringVar_Callback(self, varName, index, mode):
        #print "Ymax_Entry_StringVar_Callback varName, index, mode",varName, index, mode
        self.statusMessage.set("    Ymax_Entry_StringVar = "+self.Ymax_Entry_StringVar.get())
        #print "    new StringVar value =",self.Ymax_Entry_StringVar.get()
        
        try:
            self.ymax = float( self.Ymax_Entry_StringVar.get() )
        except:
            pass
        self.PA.set_y_max( self.ymax )
        self.plot_points()


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "Ymin_Entry_StringVar_Callback"
    def Ymin_Entry_StringVar_Callback(self, varName, index, mode):
        #print "Ymin_Entry_StringVar_Callback varName, index, mode",varName, index, mode
        self.statusMessage.set("    Ymin_Entry_StringVar = "+self.Ymin_Entry_StringVar.get())
        #print "    new StringVar value =",self.Ymin_Entry_StringVar.get()
        try:
            self.ymin = float( self.Ymin_Entry_StringVar.get() )
        except:
            pass
        self.PA.set_y_origin( self.ymin )
        self.plot_points()



    def help_popup(self):
        help_msg = """
        
Left Click = Add Point 
Right Click = Select Nearest Point 
Left Double Click List = Edit Point.
Mouse Wheel = Zoom

Esc Key = Clear Pending Actions
"""
        self.ShowInfo( title='Help', message='Mouse/Keyboard Options\r' + help_msg)

    # standard message dialogs... showinfo, showwarning, showerror
    def ShowInfo(self, title='Title', message='your message here.'):
        tkMessageBox.showinfo( title, message )
        return
    def ShowWarning(self, title='Title', message='your message here.'):
        tkMessageBox.showwarning( title, message )
        return
    def ShowError(self, title='Title', message='your message here.'):
        tkMessageBox.showerror( title, message )
        return
        
    # standard question dialogs... askquestion, askokcancel, askyesno, or askretrycancel
    # return True for OK, Yes, Retry, False for Cancel or No
    def AskYesNo(self, title='Title', message='your question here.'):
        return tkMessageBox.askyesno( title, message )
    def AskOK_Cancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askokcancel( title, message )
    def AskRetryCancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askretrycancel( title, message )
        
    # return "yes" for Yes, "no" for No
    def AskQuestion(self, title='Title', message='your question here.'):
        return tkMessageBox.askquestion( title, message )
    # END of standard message dialogs

    # >>>>>>insert any user code below this comment for section "standard_message_dialogs"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "standard_file_dialogs"
    # standard file dialogs... askdirectory, askopenfile, asksaveasfilename

    # return a string containing directory name
    def AskDirectory(self, title='Choose Directory', initialdir="."):
        dirname = tkFileDialog.askdirectory(parent=self.master,initialdir=initialdir,title=title)
        return dirname # <-- string
        
    # return an OPEN file type object OR None (opened using mode, 'r','rb','w','wb')
    # WARNING... opening file with mode 'w' or 'wb' will erase contents
    def AskOpenFile(self, title='Choose File', mode='rb', initialdir='.', filetypes=None):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]
        fileobj = tkFileDialog.askopenfile(parent=self.master,mode=mode,title=title,
            initialdir=initialdir, filetypes=filetypes)
        
        # if opened, then fileobj.name contains the name string
        return fileobj # <-- an opened file, or the value None
        
    # return a string containing file name (the calling routine will need to open the file)
    def AskSaveasFilename(self, title='Save File', filetypes=None, initialfile=''):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]

        fileName = tkFileDialog.asksaveasfilename(parent=self.master,filetypes=filetypes, initialfile=initialfile ,title=title)
        return fileName # <-- string
        

class Point( object ):
    def __init__(self, x, y):
        
        self.x = x
        self.y = y
        
    def get_str(self):
        return '%g, %g'%(self.x, self.y)
    
    def set_xy(self, x, y):
        self.x = x
        self.y = y
    
    def get_xy(self):
        return self.x, self.y
        

def main():
    """Run Main Window"""
    root = Tk()
    DigiPlot(root)
    root.mainloop()


if __name__ == '__main__':
    main()



#import base64
from Tkinter import *
from digiplot_backup_resources import file_dataD


#base64_str = file_dataD["document.thmb40.png"]
#decoded_str = base64.b64decode( base64_str.replace('\\n','') )

IMAGE_DATA = file_dataD["Cu_cte.gif"]

imgL = []

root = Tk()
for name, img64 in file_dataD.items():
    print name
    image = PhotoImage(data=img64) # ONLY works with gif and PGM/PPM images
    imgL.append( image ) # don't lose reference to image
    label = Label(root, image=image, padx=20, pady=20)
    label.pack(side=LEFT)
    
root.mainloop()







# parent_dir = D:\py_proj_2017\DigiPlot\digiplot
target_dir = r"backup"

file_typeD = {} # file type description. index=resource path, value=mime type
file_typeD[r"sample_poly.png"]="('image/png', None)"



#!/usr/bin/env python

# NOTICE... this file is generated by tk_happy.
# Any code or comments added by the user must be in designated areas ONLY.
# User additions will be lost if they are placed in code-generated areas.

# tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "imports"
from Tkinter import *
import tkMessageBox
import tkFileDialog
import tkColorChooser
# >>>>>>insert any user code below this comment for section "imports"
# Place any user import statements here

# tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "top_of_init"


class _Temp:
    def __init__(self, master):
        self.initComplete = 0
        frame = Frame(master, width=555, height=546)
        frame.pack()
        self.master = master
        self.x, self.y, self.w, self.h = -1,-1,-1,-1
        
        # bind master to <Configure> in order to handle any resizing, etc.
        # postpone self.master.bind("<Configure>", self.Master_Configure)
        self.master.bind('<Enter>', self.bindConfigure)
        

        self.master.title("temp")

        self.Plot_Canvas = Canvas(self.master,width="313", height="362")
        self.Plot_Canvas.place(x=216, y=36)
        self.Plot_Canvas.bind("<ButtonRelease-1>", self.Plot_Canvas_Click)

        self.Defined_Points_Label = Label(self.master,text="Defined Points", width="15")
        self.Defined_Points_Label.place(x=48, y=12, width=112, height=22)


        lbframe = Frame( self.master )
        self.Defined_Points_Listbox_frame = lbframe
        scrollbar = Scrollbar(lbframe, orient=VERTICAL)
        self.Defined_Points_Listbox = Listbox(lbframe, width="15", selectmode="extended", yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.Defined_Points_Listbox.yview)
        scrollbar.pack(side=RIGHT, fill=Y)
        self.Defined_Points_Listbox.pack(side=LEFT, fill=BOTH, expand=1)

        self.Defined_Points_Listbox_frame.place(x=24, y=36, width=179, height=365)
        self.Defined_Points_Listbox.bind("<ButtonRelease-1>", self.Defined_Points_Listbox_Click)
        
        # make a Status Bar
        self.statusMessage = StringVar()
        self.statusMessage.set("")
        self.statusbar = Label(self.master, textvariable=self.statusMessage, bd=1, relief=SUNKEN)
        self.statusbar.pack(anchor=SW, fill=X, side=BOTTOM)


        # >>>>>>insert any user code below this comment for section "top_of_init"
        self.statusMessage.set("Welcome to temp")
        # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menuStructure"
        self.menuBar = Menu(master, relief = "raised", bd=2)

        top_File = Menu(self.menuBar, tearoff=0)

        top_File.add("command", label = "Import Image", command = self.menu_File_Import_Image)
        top_File.add("command", label = "Save CSV", command = self.menu_File_Save_CSV)
        self.menuBar.add("cascade", label="File", menu=top_File)

        top_Anchor_Plot = Menu(self.menuBar, tearoff=0)

        top_Anchor_Plot.add("command", label = "Set Xmin", command = self.menu_Anchor_Plot_Set_Xmin)
        top_Anchor_Plot.add("command", label = "Set Xmax", command = self.menu_Anchor_Plot_Set_Xmax)
        top_Anchor_Plot.add("command", label = "Set Ymin", command = self.menu_Anchor_Plot_Set_Ymin)
        top_Anchor_Plot.add("command", label = "Set Ymax", command = self.menu_Anchor_Plot_Set_Ymax)
        self.menuBar.add("cascade", label="Anchor Plot", menu=top_Anchor_Plot)
        self.menuBar.add("command", label = "Help", command = self.menu_Help)

        top_Exit = Menu(self.menuBar, tearoff=0)

        top_Exit.add("command", label = "Quit", command = self.menu_Exit_Quit)
        self.menuBar.add("cascade", label="Exit", menu=top_Exit)

        master.config(menu=self.menuBar)

        # >>>>>>insert any user code below this comment for section "menuStructure"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_File_Import_Image"
    def menu_File_Import_Image(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_File_Import_Image"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_File_Import_Image")
        print "called menu_File_Import_Image"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_File_Save_CSV"
    def menu_File_Save_CSV(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_File_Save_CSV"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_File_Save_CSV")
        print "called menu_File_Save_CSV"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Xmin"
    def menu_Anchor_Plot_Set_Xmin(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Xmin"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Xmin")
        print "called menu_Anchor_Plot_Set_Xmin"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Xmax"
    def menu_Anchor_Plot_Set_Xmax(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Xmax"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Xmax")
        print "called menu_Anchor_Plot_Set_Xmax"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Ymin"
    def menu_Anchor_Plot_Set_Ymin(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Ymin"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Ymin")
        print "called menu_Anchor_Plot_Set_Ymin"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Anchor_Plot_Set_Ymax"
    def menu_Anchor_Plot_Set_Ymax(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Anchor_Plot_Set_Ymax"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Anchor_Plot_Set_Ymax")
        print "called menu_Anchor_Plot_Set_Ymax"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Help"
    def menu_Help(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Help"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Help")
        print "called menu_Help"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "menu_Exit_Quit"
    def menu_Exit_Quit(self):
        pass
        # >>>>>>insert any user code below this comment for section "menu_Exit_Quit"
        # replace, delete, or comment-out the following
        self.statusMessage.set("called menu_Exit_Quit")
        print "called menu_Exit_Quit"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "Master_Configure"
    def bindConfigure(self, event):
        if not self.initComplete:
            self.master.bind("<Configure>", self.Master_Configure)
            self.initComplete = 1


    def Master_Configure(self, event):
        pass
        # >>>>>>insert any user code below this comment for section "Master_Configure"
        # replace, delete, or comment-out the following
        if event.widget != self.master:
            if self.w != -1:
                return
        x = int(self.master.winfo_x())
        y = int(self.master.winfo_y())
        w = int(self.master.winfo_width())
        h = int(self.master.winfo_height())
        if (self.x, self.y, self.w, self.h) == (-1,-1,-1,-1):
            self.x, self.y, self.w, self.h = x,y,w,h


        if self.w!=w or self.h!=h:
            print "Master reconfigured... make resize adjustments"
            self.w=w
            self.h=h
    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=1"
    def Plot_Canvas_Click(self, event): #click method for component ID=1
        pass
        # >>>>>>insert any user code below this comment for section "compID=1"
        # replace, delete, or comment-out the following
        print "executed method Plot_Canvas_Click"
        self.statusMessage.set("executed method Plot_Canvas_Click")
        print "clicked in canvas at x,y =",event.x,event.y
        w = int(self.Plot_Canvas.cget("width"))
        h = int(self.Plot_Canvas.cget("height"))
        self.Plot_Canvas.create_rectangle((2, 2, w+1, h+1), outline="blue")
        self.Plot_Canvas.create_line(0, 0, w+2, h+2, fill="red")
        x = int(event.x)
        y = int(event.y)
        print "event x,y=",x,y
        self.Plot_Canvas.create_text(x,y, text="NE", fill="green", anchor=NE)
        self.Plot_Canvas.create_text(x,y, text="SW", fill="magenta", anchor=SW)

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=2"
    def Defined_Points_Listbox_Click(self, event): #click method for component ID=2
        pass
        # >>>>>>insert any user code below this comment for section "compID=2"
        # replace, delete, or comment-out the following
        print "executed method Defined_Points_Listbox_Click"
        self.statusMessage.set("executed method Defined_Points_Listbox_Click")
        print "current selection(s) =",self.Defined_Points_Listbox.curselection()
        labelL = []
        for i in self.Defined_Points_Listbox.curselection():
            labelL.append( self.Defined_Points_Listbox.get(i))
        print "current label(s) =",labelL
        # use self.Defined_Points_Listbox.insert(0, "item zero")
        #     self.Defined_Points_Listbox.insert(index, "item i")
        #            OR
        #     self.Defined_Points_Listbox.insert(END, "item end")
        #   to insert items into the list box

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "standard_message_dialogs"

    # standard message dialogs... showinfo, showwarning, showerror
    def ShowInfo(self, title='Title', message='your message here.'):
        tkMessageBox.showinfo( title, message )
        return
    def ShowWarning(self, title='Title', message='your message here.'):
        tkMessageBox.showwarning( title, message )
        return
    def ShowError(self, title='Title', message='your message here.'):
        tkMessageBox.showerror( title, message )
        return
        
    # standard question dialogs... askquestion, askokcancel, askyesno, or askretrycancel
    # return True for OK, Yes, Retry, False for Cancel or No
    def AskYesNo(self, title='Title', message='your question here.'):
        return tkMessageBox.askyesno( title, message )
    def AskOK_Cancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askokcancel( title, message )
    def AskRetryCancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askretrycancel( title, message )
        
    # return "yes" for Yes, "no" for No
    def AskQuestion(self, title='Title', message='your question here.'):
        return tkMessageBox.askquestion( title, message )
    # END of standard message dialogs

    # >>>>>>insert any user code below this comment for section "standard_message_dialogs"


    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "standard_file_dialogs"
    # standard file dialogs... askdirectory, askopenfile, asksaveasfilename

    # return a string containing directory name
    def AskDirectory(self, title='Choose Directory', initialdir="."):
        dirname = tkFileDialog.askdirectory(parent=self.master,initialdir=initialdir,title=title)
        return dirname # <-- string
        
    # return an OPEN file type object OR None (opened using mode, 'r','rb','w','wb')
    # WARNING... opening file with mode 'w' or 'wb' will erase contents
    def AskOpenFile(self, title='Choose File', mode='rb', initialdir='.', filetypes=None):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]
        fileobj = tkFileDialog.askopenfile(parent=self.master,mode=mode,title=title,
            initialdir=initialdir, filetypes=filetypes)
        
        # if opened, then fileobj.name contains the name string
        return fileobj # <-- an opened file, or the value None
        
    # return a string containing file name (the calling routine will need to open the file)
    def AskSaveasFilename(self, title='Save File', filetypes=None, initialfile=''):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]

        fileName = tkFileDialog.asksaveasfilename(parent=self.master,filetypes=filetypes, initialfile=initialfile ,title=title)
        return fileName # <-- string
        
    # END of standard file dialogs

    # >>>>>>insert any user code below this comment for section "standard_file_dialogs"


# tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "end"

def main():
    root = Tk()
    app = _Temp(root)
    root.mainloop()

if __name__ == '__main__':
    main()



import numpy
# From: https://stackoverflow.com/questions/14177744/how-does-perspective-transformation-work-in-pil

def find_coeffs(pa, pb):
    """
    pa and pb contain 4 corresponding points on image in order UL,UR,LR,LL
    UL=(0,0), UR=(w,0), LR=(w,h), LL=(0,h)
    
    Units for pa and pb are pixels.
    pa is target plane (e.g. [(0, 0), (256, 0), (256, 256), (0, 256)])
    pb is source plane (e.g. [(0, 0), (256, 0), (new_width, height), (xshift, height)])
    """
    matrix = []
    for p1, p2 in zip(pa, pb):
        matrix.append([p1[0], p1[1], 1, 0, 0, 0, -p2[0]*p1[0], -p2[0]*p1[1]])
        matrix.append([0, 0, 0, p1[0], p1[1], 1, -p2[1]*p1[0], -p2[1]*p1[1]])

    A = numpy.matrix(matrix, dtype=numpy.float)
    B = numpy.array(pb).reshape(8)

    #res = numpy.dot(numpy.linalg.inv(A.T * A) * A.T, B)
    # take commenters advice to avoid A.T
    res = numpy.linalg.solve(A, B)
    return numpy.array(res).reshape(8)

def fix_plot_img( pos_xmin, pos_xmax, pos_ymin, pos_ymax, img):
    """
    Correct plot so that xmin and xmax have same y value.
    And, ymin/ymax have same x value
    """
    xlo = pos_xmin[0]
    xhi = pox_xmax[0]
    
    # Note y reversed because 0,0 is UL
    ylo = pos_ymin[1]
    yhi = pos_ymax[1]
    
    # make target points
    UL = (xlo, yhi)
    UR = (xhi, yhi)
    LR = (xhi, ylo)
    LL = (xlo, ylo)
    pa = [UL,UR,LR,LL]
    
    # make source points
    dxlo = pos_ymax[0] - pos_ymin[0]
    dylo = pos_xmax[1] - pos_xmin[1]
    UL = (xlo+dxlo, yhi)
    LR = (xhi, ylo+dylo)
    pb = [UL,UR,LR,LL]
    
    coeffs = find_coeffs(pa, pb)
    img.transform((width, height), Image.PERSPECTIVE, coeffs, Image.BICUBIC)
    
    
    
    